@@ -44,67 +44,43 @@
 #include "nclextra/myreader.h"
 #include <sstream>
 
-string::size_type posRateHeterotachy(string &model_name) {
-	string::size_type pos1 = 0, pos2 = 0;
-    do {
-        pos1 = model_name.find("+H", pos1);
-        if (pos1 == string::npos) break;
-    } while (pos1 < model_name.length()-2 && isalpha(model_name[pos1+2]));
-
-    do {
-        pos2 = model_name.find("*H");
-        if (pos2 == string::npos) break;
-    } while (pos2 < model_name.length()-2 && isalpha(model_name[pos2+2]));
-
+string::size_type findSubStr(string &name, string sub1, string sub2) {
+    string::size_type pos1, pos2;
+    for (pos1 = 0; pos1 != string::npos; pos1++) {
+        pos1 = name.find(sub1, pos1);
+        if (pos1 == string::npos)
+            break;
+        if (pos1+2 >= name.length() || !isalpha(name[pos1+2])) {
+            break;
+        }
+    }
+    
+    for (pos2 = 0; pos2 != string::npos; pos2++) {
+        pos2 = name.find(sub2, pos2);
+        if (pos2 == string::npos)
+            break;
+        if (pos2+2 >= name.length() ||!isalpha(name[pos2+2]))
+            break;
+    }
+    
     if (pos1 != string::npos && pos2 != string::npos) {
         return min(pos1, pos2);
     } else if (pos1 != string::npos)
         return pos1;
     else
         return pos2;
+}
 
+string::size_type posRateHeterotachy(string &model_name) {
+    return findSubStr(model_name, "+H", "*H");
 }
 
 string::size_type posRateFree(string &model_name) {
-	string::size_type pos1 = 0, pos2 = 0;
-    do {
-        pos1 = model_name.find("+R", pos1);
-        if (pos1 == string::npos) break;
-    } while (pos1 < model_name.length()-2 && isalpha(model_name[pos1+2]));
-
-    do {
-        pos2 = model_name.find("*R");
-        if (pos2 == string::npos) break;
-    } while (pos2 < model_name.length()-2 && isalpha(model_name[pos2+2]));
-
-    if (pos1 != string::npos && pos2 != string::npos) {
-        return min(pos1, pos2);
-    } else if (pos1 != string::npos)
-        return pos1;
-    else
-        return pos2;
-
+    return findSubStr(model_name, "+R", "*R");
 }
 
 string::size_type posPOMO(string &model_name) {
-	string::size_type pos1 = 0, pos2 = 0;
-    do {
-        pos1 = model_name.find("+P", pos1);
-        if (pos1 == string::npos) break;
-    } while (pos1 < model_name.length()-2 && isalpha(model_name[pos1+2]));
-
-    do {
-        pos2 = model_name.find("*P");
-        if (pos2 == string::npos) break;
-    } while (pos2 < model_name.length()-2 && isalpha(model_name[pos2+2]));
-
-    if (pos1 != string::npos && pos2 != string::npos) {
-        return min(pos1, pos2);
-    } else if (pos1 != string::npos)
-        return pos1;
-    else
-        return pos2;
-
+    return findSubStr(model_name, "+P", "*P");
 }
 
 ModelsBlock *readModelsDefinition(Params &params) {
@@ -120,12 +96,21 @@ ModelsBlock *readModelsDefinition(Params &params) {
 		nexus.Add(models_block);
 	    MyToken token(in);
 	    nexus.Execute(token);
-//	    int num_model = 0, num_freq = 0;
-//	    for (ModelsBlock::iterator it = models_block->begin(); it != models_block->end(); it++)
-//	    	if ((*it).flag & NM_FREQ) num_freq++; else num_model++;
-//	    cout << num_model << " models and " << num_freq << " frequency vectors loaded" << endl;
 	} catch (...) {
-        ASSERT(0 && "predefined mixture models initialized");
+        ASSERT(0 && "predefined mixture models not initialized");
+    }
+
+	try
+	{
+		// loading internal protei model definitions
+		stringstream in(builtin_prot_models);
+        ASSERT(in && "stringstream is OK");
+		NxsReader nexus;
+		nexus.Add(models_block);
+	    MyToken token(in);
+	    nexus.Execute(token);
+	} catch (...) {
+        ASSERT(0 && "predefined protein models not initialized");
     }
 
 	if (params.model_def_file) {
@@ -136,7 +121,7 @@ ModelsBlock *readModelsDefinition(Params &params) {
 	    nexus.Execute(token);
 	    int num_model = 0, num_freq = 0;
 	    for (ModelsBlock::iterator it = models_block->begin(); it != models_block->end(); it++)
-	    	if ((*it).flag & NM_FREQ) num_freq++; else num_model++;
+	    	if (it->second.flag & NM_FREQ) num_freq++; else num_model++;
 	    cout << num_model << " models and " << num_freq << " frequency vectors loaded" << endl;
 	}
 	return models_block;
@@ -231,9 +216,10 @@ ModelFactory::ModelFactory(Params &params, string &model_name, PhyloTree *tree,
 			size_t pos = findCloseBracket(model_str, spec_pos);
 			if (pos == string::npos)
 				outError("Model name has wrong bracket notation '{...}'");
-			rate_str = model_str.substr(pos+1);
-			model_str = model_str.substr(0, pos+1);
-        } else {
+				rate_str = model_str.substr(pos+1);
+				model_str = model_str.substr(0, pos+1);
+        }
+    else {
             rate_str = model_str.substr(spec_pos);
             model_str = model_str.substr(0, spec_pos);
         }
@@ -295,7 +281,6 @@ ModelFactory::ModelFactory(Params &params, string &model_name, PhyloTree *tree,
     // models, the heterozygosity can be set separately for each model and the
     // +P{}, +GXX and +I flags should already be inside the model definition.
     if (model_str.substr(0, 3) != "MIX" && pomo) {
-
       // +P{} flag.
       p_pos = posPOMO(rate_str);
       if (p_pos != string::npos) {
@@ -360,7 +345,7 @@ ModelFactory::ModelFactory(Params &params, string &model_name, PhyloTree *tree,
 	if (freq_type == FREQ_UNKNOWN) {
 		switch (tree->aln->seq_type) {
 		case SEQ_BINARY: freq_type = FREQ_ESTIMATE; break; // default for binary: optimized frequencies
-		case SEQ_PROTEIN: freq_type = FREQ_USER_DEFINED; break; // default for protein: frequencies of the empirical AA matrix
+		case SEQ_PROTEIN: break; // let ModelProtein decide by itself
 		case SEQ_MORPH: freq_type = FREQ_EQUAL; break;
 		case SEQ_CODON: freq_type = FREQ_UNKNOWN; break;
             break;
@@ -567,6 +552,12 @@ ModelFactory::ModelFactory(Params &params, string &model_name, PhyloTree *tree,
 	if ((posasc = rate_str.find("+ASC")) != string::npos) {
 		// ascertainment bias correction
 		unobserved_ptns = tree->aln->getUnobservedConstPatterns();
+        
+        // delete rarely observed state
+        for (int i = unobserved_ptns.length()-1; i >= 0; i--)
+            if (model->state_freq[(int)unobserved_ptns[i]] < 1e-8)
+                unobserved_ptns.erase(i);
+                
 		// rebuild the seq_states to contain states of unobserved constant patterns
 		tree->aln->buildSeqStates(true);
 //		if (unobserved_ptns.size() <= 0)
@@ -1260,8 +1251,6 @@ double ModelFactory::optimizeParameters(int fixed_len, bool write_info,
         }
     }
 
-
-
 	if (verbose_mode >= VB_MED || write_info)
 		cout << "Optimal log-likelihood: " << cur_lh << endl;
 
