@@ -22,6 +22,8 @@
 #include <string.h>
 #include "modelliemarkov.h"
 #include "modelunrest.h"
+#include <Eigen/Eigenvalues>
+using namespace Eigen;
 
 
 /** number of squaring for scaling-squaring technique */
@@ -88,7 +90,7 @@ void ModelMarkov::setReversible(bool reversible) {
 
         num_params = nrate - 1;
 
-        if (phylo_tree->rooted) {
+        if (phylo_tree && phylo_tree->rooted) {
             cout << "Converting rooted to unrooted tree..." << endl;
             phylo_tree->convertToUnrooted();
         }
@@ -118,7 +120,7 @@ void ModelMarkov::setReversible(bool reversible) {
         if (!cinv_evec)
             cinv_evec = aligned_alloc<complex<double> >(num_states*num_states);
         
-        if (!phylo_tree->rooted) {
+        if (phylo_tree && !phylo_tree->rooted) {
             cout << "Converting unrooted to rooted tree..." << endl;
             phylo_tree->convertToRooted();
         }
@@ -322,8 +324,8 @@ void ModelMarkov::init(StateFreqType type) {
 
 void ModelMarkov::writeInfo(ostream &out) {
 	if (is_reversible && num_states == 4) {
-    report_rates(out, "Rate parameters", rates);
-    report_state_freqs(out);
+        report_rates(out, "Rate parameters", rates);
+        report_state_freqs(out);
 		//if (freq_type != FREQ_ESTIMATE) return;
 	} else if (!is_reversible) {
         // non-reversible
@@ -335,7 +337,7 @@ void ModelMarkov::writeInfo(ostream &out) {
 
         if (num_states != 4) return;
 		report_rates(out, "Substitution rates", rates);
-    report_state_freqs(out, state_freq);
+        report_state_freqs(out, state_freq);
     }
 }
 
@@ -947,6 +949,19 @@ void ModelMarkov::decomposeRateMatrix(){
 			}
 		}
 		delete [] q;
+/*    } else if (Params::getInstance().matrix_exp_technique == MET_EIGEN3LIB_DECOMPOSITION) {
+        // Using Eigen3 libary for general time-reversible model
+        Eigen::MatrixXd Q;
+        Q.resize(num_states, num_states);
+        if (half_matrix) {
+            for (i = 0, k = 0; i < num_states; i++)
+                for (j = i+1; j < num_states; j++, k++) {
+                    Q(i,j) = Q(j,i) = rates[k];
+                }
+        } else {
+            // full matrix
+        }
+*/
 	} else {
 
         // general reversible model
@@ -1068,13 +1083,32 @@ void ModelMarkov::readStateFreq(string str) throw(const char*) {
 		outError("State frequencies do not sum up to 1.0 in ", str);
 }
 
-void ModelMarkov::readParameters(const char *file_name) { 
+void ModelMarkov::readParameters(const char *file_name) {
+    if (!fileExists(file_name))
+        outError("File not found ", file_name);
+
+    cout << "Reading model parameters from file " << file_name << endl;
+
+    // if detect if reading full matrix or half matrix by the first entry
+	try {
+		ifstream in(file_name);
+        double d;
+        in >> d;
+        if (d < 0) {
+            setReversible(false);
+        } else
+            setReversible(true);
+        in.close();
+    }
+	catch (...) {
+		outError(ERR_READ_ANY, file_name);
+	}
+
 	try {
 		ifstream in(file_name);
 		if (in.fail()) {
 			outError("Invalid model name ", file_name);
         }
-		cout << "Reading model parameters from file " << file_name << endl;
 		readRates(in);
 		readStateFreq(in);
 		in.close();
@@ -1084,6 +1118,51 @@ void ModelMarkov::readParameters(const char *file_name) {
 	} 
 	num_params = 0;
 	writeInfo(cout);
+
+    if (!is_reversible) {
+        // check consistency of state_freq
+        double saved_state_freq[num_states];
+        memcpy(saved_state_freq, state_freq, sizeof(double)*num_states);
+        decomposeRateMatrix();
+        for (int i = 0; i < num_states; i++)
+            if (fabs(state_freq[i] - saved_state_freq[i]) > 1e-3)
+                cout << "WARNING: State " << i << " frequency " << state_freq[i]
+                     << " does not match " << saved_state_freq[i] << endl;
+    }
+}
+
+void ModelMarkov::readParametersString(string &model_str) {
+
+    // if detect if reading full matrix or half matrix by the first entry
+    int end_pos;
+    double d = 0.0;
+    d = convert_double(model_str.c_str(), end_pos);
+    if (d < 0) {
+        setReversible(false);
+    } else
+        setReversible(true);
+
+	try {
+		stringstream in(model_str);
+		readRates(in);
+		readStateFreq(in);
+	}
+	catch (const char *str) {
+		outError(str);
+	} 
+	num_params = 0;
+	writeInfo(cout);
+
+    if (!is_reversible) {
+        // check consistency of state_freq
+        double saved_state_freq[num_states];
+        memcpy(saved_state_freq, state_freq, sizeof(double)*num_states);
+        decomposeRateMatrix();
+        for (int i = 0; i < num_states; i++)
+            if (fabs(state_freq[i] - saved_state_freq[i]) > 1e-3)
+                cout << "WARNING: State " << i << " frequency " << state_freq[i]
+                     << " does not match " << saved_state_freq[i] << endl;
+    }
 }
 
 
