@@ -677,7 +677,8 @@ void MTree::readTree(istream &in, bool &is_rooted)
         DoubleVector branch_len;
         Node *node;
         parseFile(in, ch, node, branch_len);
-        if (is_rooted || !branch_len.empty()) {
+        // 2018-01-05: assuming rooted tree if root node has two children
+        if (is_rooted || !branch_len.empty() || node->degree() == 2) {
             if (branch_len.empty())
                 branch_len.push_back(-1.0);
             if (branch_len[0] == -1.0) branch_len[0] = 0.0;
@@ -837,6 +838,7 @@ void MTree::parseFile(istream &infile, char &ch, Node* &root, DoubleVector &bran
         ch = readNextChar(infile, ch);
     if (seqlen == maxlen)
         throw "Too long name ( > 1000)";
+    renameString(seqname);
 //    seqname[seqlen] = 0;
     if (seqlen == 0 && root->isLeaf())
         throw "A taxon has no name.";
@@ -1359,6 +1361,43 @@ Node *MTree::findNodeName(string &name, Node *node, Node *dad) {
     return NULL;
 }
 
+bool MTree::findNodeNames(unordered_set<string> &taxa_set, pair<Node*,Neighbor*> &res, Node *node, Node *dad) {
+    int presence = 0;
+    Neighbor *target = NULL;
+    FOR_NEIGHBOR_IT(node, dad, it) {
+        if ((*it)->node->isLeaf()) {
+            if (taxa_set.find((*it)->node->name) != taxa_set.end()) {
+                presence++;
+            } else target = (*it);
+        } else {
+            if (findNodeNames(taxa_set, res, (*it)->node, node))
+                presence++;
+            else
+                target = *it;
+            if (res.first)
+                return false;
+        }
+    }
+    // all presence or absence
+    if (presence == node->neighbors.size()-1)
+        return true;
+    if (presence == 0)
+        return false;
+    // inbetween: detect it!
+    res.first = node;
+    res.second = target;
+    if (target != node->neighbors[0]) {
+        // move target into the first neighbor
+        FOR_NEIGHBOR_IT(node, NULL, it)
+        if ((*it) == target) {
+            (*it) = node->neighbors[0];
+            node->neighbors[0] = target;
+            break;
+        }
+    }
+    return false;
+}
+
 Node *MTree::findLeafName(string &name, Node *node, Node *dad) {
     if (!node) node = root;
     if (node->isLeaf() && node->name == name) return node;
@@ -2392,8 +2431,11 @@ int MTree::removeTaxa(StrVector &taxa_names) {
 	ASSERT(taxa.size() > 0);
 	// reassign taxon IDs
 	int id = 0;
-	for (NodeVector::iterator nit = taxa.begin(); nit != taxa.end(); nit++, id++)
-		(*nit)->id = id;
+	for (NodeVector::iterator nit = taxa.begin(); nit != taxa.end(); nit++)
+        if (*nit == root && rooted)
+            (*nit)->id = taxa.size()-1;
+        else
+            (*nit)->id = id++;
 	leafNum = taxa.size();
 	initializeTree();
     return count;
