@@ -444,8 +444,11 @@ public:
      */
     virtual void setAlignment(Alignment *alignment);
 
-    /** set the root by name */
-    void setRootNode(const char *my_root);
+    /** set the root by name
+        @param my_root root node name
+        @param multi_taxa TRUE if my_root is a comma-separated list of nodes
+     */
+    void setRootNode(const char *my_root, bool multi_taxa = false);
 
 
     /**
@@ -1307,6 +1310,13 @@ public:
      */
     virtual double optimizeAllBranches(int my_iterations = 100, double tolerance = TOL_LIKELIHOOD, int maxNRStep = 100);
 
+    void moveRoot(Node *node1, Node *node2);
+
+    /**
+        Optimize root position for rooted tree
+    */
+    virtual double optimizeRootPosition(bool write_info, double logl_epsilon);
+
     /**
             inherited from Optimization class, to return to likelihood of the tree
             when the current branceh length is set to value
@@ -1661,7 +1671,7 @@ public:
     /**
             Resampling estimated log-likelihood (RELL)
      */
-    void resampleLh(double **pat_lh, double *lh_new);
+    void resampleLh(double **pat_lh, double *lh_new, int *rstream);
 
     /**
             Test one branch of the tree with aLRT SH-like interpretation
@@ -1779,6 +1789,9 @@ public:
      */
     double *var_matrix;
 
+    /** distance matrix file */
+    string dist_file;
+    
     /**
             TRUE if you want to optimize branch lengths by Newton-Raphson method
      */
@@ -1944,6 +1957,13 @@ public:
     */
     virtual void writeSiteLh(ostream &out, SiteLoglType wsl, int partid = -1);
 
+    /**
+        write branches into a csv file
+        Feature requested by Rob Lanfear
+        @param out output stream
+     */
+    virtual void writeBranches(ostream &out);
+    
 protected:
 
     /**
