@@ -1286,6 +1286,9 @@ public:
     /** number iterations for parameter optimization, default: 100 */
     int num_param_iterations;
 
+    /** number of independent runs (-nrun option) */
+    int num_runs;
+    
     /**
             name of the substitution model (e.g., HKY, GTR, TN+I+G, JC+G, etc.)
      */
@@ -1803,8 +1806,12 @@ public:
 	/** true to print all UFBoot trees to a file */
 	int print_ufboot_trees;
 
-    int contree_rfdist;
+    /**********************************************/
+    /**** variables for jackknife ******************/
 
+    /** proportion of sites to be dropped in jackknife */
+    double jackknife_prop;
+    
     /****** variables for NNI cutoff heuristics ******/
 
     /**
@@ -1869,6 +1876,9 @@ public:
 
     /** number of threads for OpenMP version     */
     int num_threads;
+    
+    /** maximum number of threads, default: #CPU scores  */
+    int num_threads_max;
 
     /** either MTC_AIC, MTC_AICc, MTC_BIC */
     ModelTestCriterion model_test_criterion;
@@ -1931,6 +1941,9 @@ public:
     /** TRUE to write initial tree to a file (default: false) */
     bool write_init_tree;
 
+    /** TRUE to write branch lengths of partition trees for each branch of supertree */
+    bool write_branches;
+    
     /** frequencies of const patterns to be inserted into alignment */
     char *freq_const_patterns;
     /** BQM 2015-02-25: true to NOT rescale Gamma+Invar rates by (1-p_invar) */
@@ -2244,6 +2257,13 @@ void convert_range(const char *str, double &lower, double &upper, double &step_s
 
 void convert_string_vec(const char *str, StrVector &str_vec);
 
+/**
+    change unusual character in names into underscore (_)
+    @param[in/out] name string name
+    @return true if renamed, false otherwise
+ */
+bool renameString(string &name);
+
 /**
         read the file containing branch/split scaling factor and taxa weights
         @param params program parameters
